package org.ow2.orchestra.env;

import java.io.InputStream;
import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;

import org.ow2.orchestra.env.binding.ChainerBinding;
import org.ow2.orchestra.env.binding.ImplementationBinding;
import org.ow2.orchestra.env.binding.OrchestraRetryInterceptorBinding;
import org.ow2.orchestra.env.binding.QuerierDbSessionBinding;
import org.ow2.orchestra.env.binding.QuerierListAccessorBinding;
import org.ow2.orchestra.env.binding.RuntimeDbSessionBinding;
import org.ow2.orchestra.pvm.env.EnvironmentFactory;
import org.ow2.orchestra.pvm.internal.env.PvmEnvironmentFactoryParser;
import org.ow2.orchestra.pvm.internal.xml.Bindings;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.util.Misc;
import org.xml.sax.InputSource;

public class BpelEnvironmentParser extends PvmEnvironmentFactoryParser {

  private static final String ENVIRONMENT_ORCHESTRA_XSD = "environment-orchestra.xsd";

  private static PvmEnvironmentFactoryParser instance;

  public static final String CLASSPATH_URL = "classpath://";

  public static synchronized PvmEnvironmentFactoryParser getInstance() {
    if (BpelEnvironmentParser.instance == null) {
      BpelEnvironmentParser.instance = new BpelEnvironmentParser();
      // Get bindings
      final Bindings bindings = BpelEnvironmentParser.instance.getEnvironmentXmlParser().getBindings();
      // Use the same bindings in block and application contexts
      BpelEnvironmentParser.instance.getEnvironmentFactoryXmlParser().setBindings(bindings);
      // Add orchestra bindings
      bindings.addBinding(new RuntimeDbSessionBinding());
      bindings.addBinding(new QuerierDbSessionBinding());
      bindings.addBinding(new ImplementationBinding("repository"));
      bindings.addBinding(new ImplementationBinding("invoker"));
      bindings.addBinding(new ImplementationBinding("publisher"));
      bindings.addBinding(new ImplementationBinding("recorder"));
      bindings.addBinding(new ImplementationBinding("journal"));
      bindings.addBinding(new ImplementationBinding("finished-instance-handler"));
      bindings.addBinding(new ImplementationBinding("undeployed-process-handler"));
      bindings.addBinding(new ChainerBinding());
      bindings.addBinding(new ImplementationBinding("history"));
      bindings.addBinding(new ImplementationBinding("archiver"));
      bindings.addBinding(new QuerierListAccessorBinding());
      bindings.addBinding(new OrchestraRetryInterceptorBinding());
    }
    return BpelEnvironmentParser.instance;
  }

  public static EnvironmentFactory parseEnvironmentFactoryFromXmlString(final String xmlString) {
    final Parse parse = BpelEnvironmentParser.getInstance().createParse();
    final EnvironmentFactory factory = (EnvironmentFactory) parse.setString(xmlString).execute().getDocumentObject();
    parse.checkProblems("environment");
    return factory;
  }

  public static synchronized EnvironmentFactory parseEnvironmentFactoryFromURL(final URL url) {
    final Parse parse = BpelEnvironmentParser.getInstance().createParse();
    final EnvironmentFactory factory = (EnvironmentFactory) parse.setUrl(url).execute().getDocumentObject();
    parse.checkProblems("environment");
    return factory;
  }

  public static synchronized EnvironmentFactory parseEnvironmentFactoryFromResource(final String resource) {
    Misc.checkArgsNotNull(resource);
    final Parse parse = BpelEnvironmentParser.getInstance().createParse();
    final EnvironmentFactory factory = (EnvironmentFactory) parse.setResource(resource).execute().getDocumentObject();
    parse.checkProblems("environment");
    return factory;
  }

  @Override
  public synchronized DocumentBuilderFactory getDocumentBuilderFactory() {
    this.documentBuilderFactory = this.newDocumentBuilderFactory();
    this.documentBuilderFactory.setNamespaceAware(true);
    this.documentBuilderFactory.setValidating(true);
    // ignore white space can only be set if parser is validating
    this.documentBuilderFactory.setIgnoringElementContentWhitespace(true);
    // select xml schema as the schema language (a.o.t. DTD)
    this.documentBuilderFactory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
    // set schema sources
    final URL url = Thread.currentThread().getContextClassLoader().getResource(BpelEnvironmentParser.ENVIRONMENT_ORCHESTRA_XSD);
    Misc.badStateIfNull(url, "Ouch! Can't get the Orchestra environment XML Schema. " + "I was looking for a resource called: "
        + BpelEnvironmentParser.ENVIRONMENT_ORCHESTRA_XSD + " which is usally provided within the Orchestra distribution.");
    this.documentBuilderFactory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaSource", url.toExternalForm());
    return this.documentBuilderFactory;
  }

  /*
   * (non-Javadoc)
   *
   * @see org.ow2.orchestra.xml.Parser#resolveEntity(java.lang.String,
   * java.lang.String)
   */
  @Override
  public InputSource resolveEntity(final String publicId, final String systemId) {
    if (systemId.startsWith(BpelEnvironmentParser.CLASSPATH_URL)) {
      final String localpart = systemId.substring(BpelEnvironmentParser.CLASSPATH_URL.length());
      final InputStream inStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(localpart);
      if (inStream != null) {
        return new InputSource(inStream);
      }
    }
    return super.resolveEntity(publicId, systemId);
  }
}
