/**
 * Copyright (C) 2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.exception;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.var.Message;

/**
 * Represents a fault resulting from an Invoke.
 */
public class FaultWithMessage extends BpelFaultException {

  private static final long serialVersionUID = 2381218713694655654L;

  protected Message faultMessage;

  // for persistence
  protected FaultWithMessage() {
  }

  public FaultWithMessage(final QName faultName, final Message faultMessage) {
    super(faultName);
    if (faultMessage != null && !faultMessage.getParts().isEmpty()) {
      this.faultMessage = faultMessage.duplicate();
    }
  }

  public Message getFaultMessage() {
    return this.faultMessage;
  }

  @Override
  public String toString() {
    return "Fault " + this.faultName + (this.faultMessage != null ? " (data: " + this.faultMessage + ")." : ".");
  }

  @Override
  public FaultWithMessage copy() {
    return new FaultWithMessage(this.getFaultName(), this.getFaultMessage());
  }
}
