/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx;

import java.lang.management.ManagementFactory;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.ow2.orchestra.env.BpelEnvironmentParser;
import org.ow2.orchestra.env.EnvFactoryRepository;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.facade.jmx.RemoteDeployer;
import org.ow2.orchestra.facade.jmx.RemoteDeployerMBean;
import org.ow2.orchestra.pvm.env.EnvironmentFactory;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobExecutor;

public class JMXAgent implements ServletContextListener {

  private MBeanServer mbs;

  private ObjectName objectName;

  private JMXConnectorServer cs;

  private final JMXConfiguration configuration = JMXConfiguration.getInstance();

  // Default environment for deployed services
  public static String defaultEnvConfigFile = "environment.xml";

  public void contextInitialized(final ServletContextEvent arg0) {
    try {
      final EnvironmentFactory envFactory = BpelEnvironmentParser
          .parseEnvironmentFactoryFromResource(JMXAgent.defaultEnvConfigFile);
      EnvFactoryRepository.set(envFactory);
    } catch (final Exception e) {
      throw new OrchestraRuntimeException("Problem creating the environment", e);
    }
    final JobExecutor jobExecutor = EnvFactoryRepository.get().get(JobExecutor.class);
    if (jobExecutor != null) {
      jobExecutor.start();
    }
    try {
      if (System.getSecurityManager() == null) {
        System.setSecurityManager(new SecurityManager());
      }
      // start rmiregistry
      java.rmi.registry.LocateRegistry.createRegistry(this.configuration.getPort());

      this.mbs = ManagementFactory.getPlatformMBeanServer();
      final RemoteDeployerMBean remoteDeployer = new RemoteDeployer();
      // Uniquely identify the MBeans and register them with the MBeanServer
      this.objectName = ObjectName.getInstance(this.configuration.getObjectName());
      this.mbs.registerMBean(remoteDeployer, this.objectName);
      final JMXServiceURL url = new JMXServiceURL(this.configuration.getServiceUrl());
      this.cs = JMXConnectorServerFactory.newJMXConnectorServer(url, null, this.mbs);
      this.cs.start();

    } catch (final Exception e) {
      throw new OrchestraRuntimeException("Problem registering mbean in mbean server",
          e);
    }
  }

  public void contextDestroyed(final ServletContextEvent arg0) {
    if (this.mbs != null) {
      try {
        this.cs.stop();
        this.mbs.unregisterMBean(this.objectName);
      } catch (final Exception e) {
        throw new OrchestraRuntimeException("Problem unregistering mbean", e);
      }
    }
    if (EnvFactoryRepository.get() != null) {
      final JobExecutor jobExecutor = EnvFactoryRepository.get().get(JobExecutor.class);
      if (jobExecutor != null) {
        jobExecutor.stop();
      }
      EnvFactoryRepository.get().close();
      EnvFactoryRepository.set(null);
    }
  }

}
