package org.ow2.orchestra.jmx;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.ow2.orchestra.facade.AccessorUtil;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.util.Misc;

/**
 * JMXConfiguration. Keeps all configuration parameters for the Jmx agent.
 *
 * @author Goulven Le Jeune, Charles Souillard, Candy Chlad, Stanislas Giraudet
 *         De Boudemange, Guillaume Porcher
 *
 */
public class JMXConfiguration {

  // Axis classes are run from WEB-INF/classes/, the jmx configuration file is
  // in WEB-INF/
  private static String jmxConfigFile = "orchestra-jmx-conf.xml";

  private static JMXConfiguration configuration;

  private int port = 9999;

  private String objectName = AccessorUtil.OBJECT_NAME;

  private String serviceUrl = AccessorUtil.SERVICE_URL;

  public static synchronized JMXConfiguration getInstance() {
    if (JMXConfiguration.configuration == null) {
      InputStream input = null;
      try {
        final URL url = new URL(JMXConfiguration.jmxConfigFile);
        input = url.openStream();
      } catch (final MalformedURLException e) {
        final ClassLoader classLoader = JMXConfiguration.class.getClassLoader();
        input = classLoader.getResourceAsStream(JMXConfiguration.jmxConfigFile);
      } catch (final IOException e) {
        throw new OrchestraRuntimeException("Exception while loading JMX config from " + JMXConfiguration.jmxConfigFile, e);
      }
      if (input != null) {
        try {
          final JMXConfiguration configuration = new JMXConfiguration();
          new JMXConfigurationFileParser().parseAndSetObject(configuration, input);
          JMXConfiguration.configuration = configuration;
        } finally {
          Misc.close(input);
        }
      }
    }
    return JMXConfiguration.configuration;
  }


  public static String getJmxConfigFile() {
    return JMXConfiguration.jmxConfigFile;
  }


  public static void setJmxConfigFile(final String jmxConfigFile) {
    JMXConfiguration.jmxConfigFile = jmxConfigFile;
  }

  /**
   * Returns the configuration parameters in a string.
   *
   * @return the string representing the configuration parameters
   */
  @Override
  public String toString() {
    String s = "\n";
    s += "JMXConfiguration:" + "\n\t port: " + this.port + "\n\t objectName: "
        + this.objectName + "\n\t serviceUrl: " + this.serviceUrl + "\n";
    return s;
  }

  /**
   * @return the port
   */
  public int getPort() {
    return this.port;
  }

  /**
   * @param port
   *            the port to set
   */
  public void setPort(final String port) {
    this.port = Integer.parseInt(port);
  }

  /**
   * @return the objectName
   */
  public String getObjectName() {
    return this.objectName;
  }

  /**
   * @param objectName
   *            the objectName to set
   */
  public void setObjectName(final String objectName) {
    this.objectName = objectName;
  }

  /**
   * @return the serviceUrl
   */
  public String getServiceUrl() {
    return this.serviceUrl;
  }

  /**
   * @param serviceUrl
   *            the serviceUrl to set
   */
  public void setServiceUrl(final String serviceUrl) {
    this.serviceUrl = serviceUrl;
  }

}
