/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

/**
 *
 */
package org.ow2.orchestra.services;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.var.Message;

class AssociateMessageCommand implements Command<ExecElementToSignal> {
  private final OperationKey operationKey;
  private final Message incomingMessage;
  private final MessageCarrier messageCarrier;

  public AssociateMessageCommand(final OperationKey operationKey,
      final Message incomingMessage, final MessageCarrier messageCarrier) {
    this.operationKey = operationKey;
    this.incomingMessage = incomingMessage;
    this.messageCarrier = messageCarrier;
  }

  public ExecElementToSignal execute(final Environment environment) throws Exception {
//    MutexRepository.lockProcess(this.operationKey.getProcessQName());

    final BpelProcess bpelProcess = EnvTool.getRepository().lockProcess(this.operationKey.getProcessQName());

    final Receiver receiver = bpelProcess.getReceiver(this.operationKey);
    if (receiver == null) {
      throw new OrchestraRuntimeException("impossible to find a receiver corresponding to: " + this.operationKey);
    }
    final PendingMessage pendingMessage = receiver.storeIncomingMessage(this.incomingMessage, this.messageCarrier);

    final ExecElementToSignal exec =  receiver.associatePendingMessage(pendingMessage, bpelProcess);
    if (exec != null) {
      exec.getBpelExecution().getProcessInstance().getDbid();
    }
    return exec;
  }
}