/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.services;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobDbSession;
import org.ow2.orchestra.pvm.session.MessageSession;
import org.ow2.orchestra.runtime.BpelExecution;


public class AsyncRemovePendingMessage extends BpelMessageImpl<Boolean> {

  private PendingMessage pendingMessage;

  protected AsyncRemovePendingMessage() { };

  public AsyncRemovePendingMessage(final BpelExecution instance, final PendingMessage pendingMessage) {
    this.execution = instance;
    this.pendingMessage = pendingMessage;
  }

  public Boolean execute(final Environment environment) throws Exception {
    new RemovePendingMessageCommand(this.pendingMessage).execute(environment);
    if (this.execution != null && !this.execution.isEnded()) {
      environment.get(MessageSession.class).send(new AsyncFindExecToSignal((BpelExecution) this.execution, this.processQName));
    }

    final JobDbSession dbSession = environment.get(JobDbSession.class);
    if (dbSession == null) {
      throw new OrchestraRuntimeException("no " + JobDbSession.class.getName() + " in environment");
    }
    dbSession.delete(this);
    return true;
  }
}
