/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.services;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobDbSession;
import org.ow2.orchestra.pvm.session.MessageSession;
import org.ow2.orchestra.runtime.BpelExecution;


public class AsyncSignalExec extends BpelMessageImpl<Boolean> {

  private ReceivingElement recvElt;
  private PendingMessage pendingMessage;

  protected AsyncSignalExec() { };

  public AsyncSignalExec(final BpelExecution instance, final QName processQName, final ExecElementToSignal execElementToSignal) {
    this.processInstance = instance;
    this.processQName = processQName;
    this.execution = execElementToSignal.getBpelExecution();
    this.recvElt = execElementToSignal.getReceivingElement();
    this.pendingMessage = execElementToSignal.getPendingMessage();
  }

  public Boolean execute(final Environment environment) throws Exception {


    final BpelExecution instance = (BpelExecution) this.getExecution().getProcessInstance();
    final ExecElementToSignal execElt = new ExecElementToSignal((BpelExecution) this.getExecution(), this.recvElt, this.pendingMessage);
    if(! new SignalExecCommand(execElt).execute(environment)) {
      final ExecElementToSignal newExecElt = new ReAssociateMessageCommand(execElt).execute(environment);
      if (newExecElt!= null) {
        environment.get(MessageSession.class).send(
            new AsyncSignalExec(
                newExecElt.getBpelExecution().getProcessInstance(),
                newExecElt.getBpelExecution().getProcessDefinition().getQName(),
                newExecElt));
      }
    } else {
      environment.get(MessageSession.class).send(new AsyncRemovePendingMessage(instance.isEnded() ? null : instance, this.pendingMessage));
    }

    final JobDbSession dbSession = environment.get(JobDbSession.class);
    if (dbSession == null) {
      throw new OrchestraRuntimeException("no " + JobDbSession.class.getName() + " in environment");
    }
    dbSession.delete(this);
    return true;
  }
}
