/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

/**
 *
 */
package org.ow2.orchestra.services;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.services.itf.Repository;

class ReAssociateMessageCommand implements Command<ExecElementToSignal> {
  private final ExecElementToSignal execToSignal;

  public ReAssociateMessageCommand(final ExecElementToSignal execElementToSignal) {
    this.execToSignal = execElementToSignal;
  }

  public ExecElementToSignal execute(final Environment environment) throws Exception {
    final Repository repository = environment.get(Repository.class);
    final ReceivingElement receivingElement = repository.getUpdatedReceivingElement(this.execToSignal.getReceivingElement());
    final OperationKey operationKey = receivingElement.getOperationKey();

    final BpelProcess bpelProcess = repository.lockProcess(operationKey.getProcessQName());

    final PendingMessage pendingMessage = repository.getUpdatedPendingMessage(this.execToSignal.getPendingMessage());
    if (pendingMessage == null) {
      // message already associated => do nothing
      return null;
    }

    final Receiver receiver = bpelProcess.getReceiver(operationKey);
    if (receiver == null) {
      throw new OrchestraRuntimeException("impossible to find a receiver corresponding to: " + operationKey);
    }
    final ExecElementToSignal res = receiver.associatePendingMessage(pendingMessage, bpelProcess);
    if (res != null) {
      res.getBpelExecution().getProcessInstance().getDbid();
    }
    return res;
  }
}