/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

/**
 *
 */
package org.ow2.orchestra.services;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.services.itf.Repository;
import org.ow2.orchestra.util.EnvTool;

class RemovePendingMessageCommand implements Command<Void> {
  private PendingMessage pendingMessage;

  public RemovePendingMessageCommand(final PendingMessage pendingMessage) {
    this.pendingMessage = pendingMessage;
  }

  public Void execute(final Environment environment) throws Exception {
    final Repository repository = EnvTool.getRepository();
    this.pendingMessage = repository.getUpdatedPendingMessage(this.pendingMessage);
    final BpelProcess bpelProcess = repository.getProcess(this.pendingMessage.getOperationKey().getProcessQName());

    final Receiver receiver = bpelProcess.getReceiver(this.pendingMessage.getOperationKey());
    if (receiver == null) {
      throw new OrchestraRuntimeException("impossible to find a receiver corresponding to: " + this.pendingMessage.getOperationKey());
    }
    receiver.removePendingMessage(this.pendingMessage);
    return null;
  }
}