/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

/**
 *
 */
package org.ow2.orchestra.services;

import java.util.HashMap;
import java.util.Map;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.itf.Repository;

class SignalExecCommand implements Command<Boolean> {
  private final ExecElementToSignal execToSignal;

  public SignalExecCommand(final ExecElementToSignal execToSignal) {
    this.execToSignal = execToSignal;
  }

  public Boolean execute(final Environment environment) throws Exception {
    final Repository repository = environment.get(Repository.class);
    final BpelExecution exec = repository.getUpdatedExecution(this.execToSignal.getBpelExecution());
    final PendingMessage pendingMessage = repository.getUpdatedPendingMessage(this.execToSignal.getPendingMessage());
    final ReceivingElement receivingElement = repository.getUpdatedReceivingElement(this.execToSignal.getReceivingElement());
    if (exec != null
        && pendingMessage != null
        && Receiver.csMatch(pendingMessage.getMessage(), receivingElement, exec)) {
      final BpelProcess bpelProcess = exec.getProcessDefinition();
      final Receiver receiver = bpelProcess.getReceiver(
          receivingElement.getOperationKey());
      final MessageCarrier messageCarrier =
        MessageCarrierRepository
          .removePendingMessageCarrier(pendingMessage.getMessageCarrierId());
      receiver.addReplier(exec, receivingElement, messageCarrier);

      final Map<String, Object> parameters = new HashMap<String, Object>();
      parameters.put("receivingElement", receivingElement);
      parameters.put("message", pendingMessage.getMessage().duplicate());
      exec.signal(parameters);
      return true;
    } else {
      if (pendingMessage != null) {
        // redistribute message
        pendingMessage.setReserved(false);
      }
      if (exec != null) {
        exec.getProcessInstance().addWaitingExecution(exec);
      }
      return false;
    }
  }
}